import { isEmpty } from "@/libs/tools"
//验证器
/**  是否为空(包括数组，数字，字符串,undefined,null) */
export const isNotNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback(new Error("不能为空!"))
   }
   callback()
}
/** 必须为数字 */
export const isNum = (rule: any, value: any, callback: any) => {
   if (/^[-+]?\d+$/.test(value)) {
      callback()
   }
   callback(new Error("必须为数字!"))
}
/** 必须为数字或空 */
export const isNumOrNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (/^[-+]?\d+$/.test(value)) {
      callback()
   }
   callback(new Error("必须为数字或者空!"))
}
/** 必须为数字或者小数 */
export const isNumOrDecimal = (rule: any, value: any, callback: any) => {
   if (/^[+-]?(0|([1-9]\d*))(\.\d+)?$/g.test(value)) {
      callback()
   }
   callback(new Error("必须为数字或者小数!"))
}
/** 必须为数字或小数或空 */
export const isNumOrNullOrDecimal = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (/^[+-]?(0|([1-9]\d*))(\.\d+)?$/g.test(value)) {
      callback()
   }
   callback(new Error("必须为数字或小数或空!"))
}
/** 密码验证 必填*/
export const isPaw = (rule: any, value: any, callback: any) => {
   if (/^(?![a-zA-z]+$)(?!\d+$)(?![!@#$.,?()+\-=<>~%^&*]+$)[a-zA-Z\d!!@#$?.,()+\-=<>~%^&*]{8,20}$/.test(value)) {
      callback()
   }
   callback(new Error("格式有误，字母、数字、符号两种以上组合，8-20位!"))
}
/** 手机验证 必填*/
export const isMobile = (rule: any, value: any, callback: any) => {
   if (/^(?:(?:\+|00)86)?1[3-9]\d{9}$/.test(value)) {
      callback()
   }
   callback(new Error("必须为手机格式!"))
}
/** 手机验证 可为空*/
export const isMobileOrNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (/^(?:(?:\+|00)86)?1[3-9]\d{9}$/.test(value)) {
      callback()
   }
   callback(new Error("必须为手机格式或者空!"))
}
/** 电话验证 必填*/
export const isPhone = (rule: any, value: any, callback: any) => {
   if (/^(?:(?:\d{3}-)?\d{8}|^(?:\d{4}-)?\d{7,8})(?:-\d+)?$/.test(value)) {
      callback()
   }
   callback(new Error("必须为电话格式!"))
}
/** 电话验证 可为空*/
export const isPhoneOrNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (/^(?:(?:\d{3}-)?\d{8}|^(?:\d{4}-)?\d{7,8})(?:-\d+)?$/.test(value)) {
      callback()
   }
   callback(new Error("必须为电话格式或者空!"))
}
/** 身份证号 必填*/
export const isIDCard = (rule: any, value: any, callback: any) => {
   if (
      /^\d{6}((((((19|20)\d{2})(0[13-9]|1[012])(0[1-9]|[12]\d|30))|(((19|20)\d{2})(0[13578]|1[02])31)|((19|20)\d{2})02(0[1-9]|1\d|2[0-8])|((((19|20)([13579][26]|[2468][048]|0[48]))|(2000))0229))\d{3})|((((\d{2})(0[13-9]|1[012])(0[1-9]|[12]\d|30))|((\d{2})(0[13578]|1[02])31)|((\d{2})02(0[1-9]|1\d|2[0-8]))|(([13579][26]|[2468][048]|0[048])0229))\d{2}))(\d|X|x)$/.test(
         value
      )
   ) {
      callback()
   }
   callback(new Error("必须为身份证号格式!"))
}
/** 身份证号 可为空*/
export const isIDCardOrNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (
      /^\d{6}((((((19|20)\d{2})(0[13-9]|1[012])(0[1-9]|[12]\d|30))|(((19|20)\d{2})(0[13578]|1[02])31)|((19|20)\d{2})02(0[1-9]|1\d|2[0-8])|((((19|20)([13579][26]|[2468][048]|0[48]))|(2000))0229))\d{3})|((((\d{2})(0[13-9]|1[012])(0[1-9]|[12]\d|30))|((\d{2})(0[13578]|1[02])31)|((\d{2})02(0[1-9]|1\d|2[0-8]))|(([13579][26]|[2468][048]|0[048])0229))\d{2}))(\d|X|x)$/.test(
         value
      )
   ) {
      callback()
   }
   callback(new Error("必须为身份证号格式或者空!"))
}
/** ip地址 必填*/
export const isIp = (rule: any, value: any, callback: any) => {
   if (
      /^((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.){3}(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])(?::(?:[0-9]|[1-9][0-9]{1,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?$/.test(
         value
      )
   ) {
      callback()
   }
   callback(new Error("必须ip地址格式!"))
}
/** ip地址 可为空*/
export const isIpOrNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (
      /^((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.){3}(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])(?::(?:[0-9]|[1-9][0-9]{1,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?$/.test(
         value
      )
   ) {
      callback()
   }
   callback(new Error("必须为ip地址格式或者空!"))
}
/** 邮箱 必填*/
export const isEmail = (rule: any, value: any, callback: any) => {
   if (
      /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.test(
         value
      )
   ) {
      callback()
   }
   callback(new Error("必须邮箱格式!"))
}
/** 邮箱 可为空*/
export const isEmailOrNull = (rule: any, value: any, callback: any) => {
   if (isEmpty(value)) {
      callback()
   }
   if (
      /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.test(
         value
      )
   ) {
      callback()
   }
   callback(new Error("必须为邮箱格式或者空!"))
}
